;;; -*- Package: User; Mode: Lisp; Syntax: Common-Lisp; Base: 10. -*-
;;;
;;;	File:		TimeLogic.lisp
;;;	Author:		Johannes A. G. M. Koomen
;;;	Purpose:	TimeLogic package and system definitions
;;;	Last Edit:	02/11/89 16:34:31
;;;
;;;	Copyright (c) 1989  University of Rochester
;;;
;;;	The TimeLogic System is being made available by the University of
;;;	Rochester for research purposes.  No commercial use or distribution to
;;;	third parties is allowed without the explicit written permission of
;;;	the University of Rochester.
;;;
;;;	The University of Rochester will have a non-exclusive right, at no
;;;	expense, to the derivative works, modifications and enhancements made
;;;	to or resulting from the TimeLogic System, and the University of
;;;	Rochester shall be informed of such development and furnished with the
;;;	source codes to such works, modifications and enhancements when
;;;	available.  The University of Rochester will accept such derivative
;;;	works, modifications and enhancements "as is."
;;;
;;;	For documentation on this implementation see Technical Report #231,
;;;	Department of Computer Science, University of Rochester.
;;;

#+Symbolics
(defpackage TimeLogic (:nicknames tl) (:use lisp scl))
#+Explorer
(defpackage TimeLogic (:nicknames tl) (:use lisp ticl))
#-(or Symbolics Explorer)
(or (find-package 'TimeLogic)
    (make-package 'TimeLogic :nicknames '("TL")))

#+explorer
(defsystem timelogic
  (:name		"TimeLogic Temporal Reasoning System")
  (:short-name		"TimeLogic")
  (:pathname-default	"timelogic:timelogic;")
  (:patchable           )
  (:initial-status	:experimental)

  (:module defs1 ("tl-defs"))
  (:module defs2 ("tl-rlink-codes" "tl-dlink-codes"))
  (:module tlsys ("tl-main" "tl-contexts" "tl-rlinks" "tl-dlinks" "tl-trace"))
  (:module demos ("tl-demos"))

  (:compile-load defs1)
  (:compile-load defs2 (:fasload defs1))
  (:compile-load tlsys (:fasload defs1 defs2))
  (:compile-load demos (:fasload tlsys))
)

#+symbolics
(defsystem timelogic
    (:pretty-name	"TimeLogic Temporal Reasoning System"
     :short-name	"TimeLogic"
     :default-pathname	"timelogic:timelogic;"
     :journal-directory	"timelogic:patch;"
     :patchable		t
     :initial-status	:released
     :bug-reports	("TimeLogic-Support"
			 "Report problems with the TimeLogic Temporal Reasoning System.")
     :advertised-in	(:herald :finger :disk-label)
     :maintaining-sites	(:cs-rochester)
     :distribute-binaries t
     :initializations	nil
    )
  (:module defs1 ("tl-defs"))
  (:module defs2 ("tl-rlink-codes" "tl-dlink-codes")
	   (:uses-definitions-from defs1))
  (:module tlsys ("tl-main" "tl-contexts" "tl-rlinks" "tl-dlinks" "tl-trace")
	   (:uses-definitions-from defs2))
  (:module demos ("tl-demos")
;	   (:type :lisp-example)
;	   (:in-order-to :compile (:load tlsys))
	   (:uses-definitions-from tlsys)
	   )
  (:serial defs1 defs2 tlsys demos)
)
